"use strict";
/**
 * @module Message
 */
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var verifier_1 = require("./dsl/verifier");
var logger_1 = require("./common/logger");
var pact_node_1 = require("@pact-foundation/pact-node");
var express = require("express");
var http = require("http");
var es6_promise_1 = require("es6-promise");
var bodyParser = require("body-parser");
/**
 * A Message Provider is analagous to Consumer in the HTTP Interaction model.
 *
 * It is the initiator of an interaction, and expects something on the other end
 * of the interaction to respond - just in this case, not immediately.
 */
var MessageProvider = /** @class */ (function () {
    function MessageProvider(config) {
        this.config = config;
        this.state = {};
        if (!lodash_1.isEmpty(config.logLevel)) {
            pact_node_1.default.logLevel(config.logLevel);
        }
    }
    /**
     * Verify a Message Provider.
     */
    MessageProvider.prototype.verify = function () {
        logger_1.logger.info("Verifying message");
        // Start the verification CLI proxy server
        var app = this.setupProxyApplication();
        var server = this.setupProxyServer(app);
        // Run the verification once the proxy server is available
        return this
            .waitForServerReady(server)
            .then(this.runProviderVerification());
    };
    // Listens for the server start event
    // Converts event Emitter to a Promise
    MessageProvider.prototype.waitForServerReady = function (server) {
        return new es6_promise_1.Promise(function (resolve, reject) {
            server.on("listening", function () { return resolve(server); });
            server.on("error", function () { return reject(); });
        });
    };
    // Run the Verification CLI process
    MessageProvider.prototype.runProviderVerification = function () {
        var _this = this;
        return function (server) {
            var opts = __assign({}, (lodash_1.omit(_this.config, "handlers")), { providerBaseUrl: "http://localhost:" + server.address().port });
            // Run verification
            return new verifier_1.Verifier().verifyProvider(opts);
        };
    };
    // Get the API handler for the verification CLI process to invoke on POST /*
    MessageProvider.prototype.setupVerificationHandler = function () {
        var _this = this;
        return function (req, res) {
            // Extract the message request from the API
            var message = req.body;
            // Invoke the handler, and return the JSON response body
            // wrapped in a Message
            _this
                .findHandler(message)
                .then(function (handler) { return handler(message); })
                .then(function (o) { return res.json({ content: o }); })
                .catch(function (e) { return res.status(500).send(e); });
        };
    };
    // Get the Proxy we'll pass to the CLI for verification
    MessageProvider.prototype.setupProxyServer = function (app) {
        return http.createServer(app).listen();
    };
    // Get the Express app that will run on the HTTP Proxy
    MessageProvider.prototype.setupProxyApplication = function () {
        var app = express();
        app.use(bodyParser.json());
        app.use(bodyParser.urlencoded({
            extended: true,
        }));
        app.use(function (req, res, next) {
            res.header("Content-Type", "application/json; charset=utf-8");
            next();
        });
        // Proxy server will respond to Verifier process
        app.all("/*", this.setupVerificationHandler());
        return app;
    };
    // Lookup the handler based on the description, or get the default handler
    MessageProvider.prototype.findHandler = function (message) {
        var handler = this.config.handlers[message.description || ""];
        if (!handler) {
            logger_1.logger.warn("no handler found for message " + message.description);
            return es6_promise_1.Promise.reject("No handler found for message \"" + message.description + "\"." +
                " Check your \"handlers\" configuration");
        }
        return es6_promise_1.Promise.resolve(handler);
    };
    return MessageProvider;
}());
exports.MessageProvider = MessageProvider;
//# sourceMappingURL=messageProvider.js.map